unit u_display;

{$mode objfpc}{$H+}

interface
uses
  Classes, SysUtils, Graphics;

type tTrace = array[0..1800] of real;

const nPoints: integer = 1000;  // 1800 for WideScreen
const Divs: integer = 10;  // 18 for WideScreen

var
  Trace_1, Trace_2, Trace_3, Trace_4, Trace_5, Trace_6, Trace_7, Trace_8, Trace_9, Trace_10, Trace_11: Ttrace;
  Scale_2, Scale_3, Scale_4, Scale_5, Scale_6, Scale_7, Scale_8, Scale_9, Scale_10, Scale_11: real;
  Offset_2, Offset_3, Offset_4, Offset_5, Offset_6, Offset_7, Offset_8, Offset_9, Offset_10, Offset_11: integer;

procedure LinGraph_Init;
procedure LinGraph_DispTrace(Trace: tTrace; Color: integer);
procedure LinGraph_Clear;
procedure LinGraph_Refresh;
procedure LinGraph_Update;

implementation
uses u_main;

procedure LinGraph_Init;
var
  I, X, Y: integer;
begin
  With FrmMain.LinGraph do
  begin
    Height:= 370;
    Width:= 1810;
    With Canvas do
    begin
      Brush.Color:= clBlack;
      Pen.Color:= clWhite;
      Font.Color:= clWhite;
      AutoReDraw:= true;
      Pen.Style := psSolid;
      Rectangle (0, 0, nPoints + 10, 370);
    end;
    For I:= 0 To Divs do // vertical lines
    begin
      X:= 5 + I * 100; //
      Canvas.Line (X, 5, X, 365);
    end;
    For I:= 0 To 8 do // 9 horizontal lines
    begin
      Y:= 365 - I * 45;
      Canvas.Line (5, Y, nPoints + 5, Y);
      Canvas.TextOut (8, Y - 5, format ('%3d', [I * 45]));
    end;
  end;
end;

procedure LinGraph_DispTrace (Trace: tTrace; Color: integer);
var
X, Y: integer;
begin
  With FrmMain.LinGraph.Canvas do
  begin
    Pen.Color:= Color;
    Pen.Style := psSolid;
    For X:= 0 To nPoints - 1 do
    begin
      Y:= round (Trace[X]);
      // clip the graph
      If Y < 0 Then Y:= 0;
      If Y > 360 Then Y:= 360;
      if X = 0 then Moveto (X + 5, 370 - Y) else Lineto (X + 5, 370 - Y);
    end;
  end;
end;

procedure LinGraph_Refresh;
begin
with FrmMain do
  begin
  if CbxTrace1.Checked then LinGraph_DispTrace (Trace_1, clYellow);
  if CbxTrace2.Checked then LinGraph_DispTrace (Trace_2, clRed);
  if CbxTrace3.Checked then LinGraph_DispTrace (Trace_3, clLime);
  if CbxTrace4.Checked then LinGraph_DispTrace (Trace_4, clFuchsia);
  if CbxTrace5.Checked then LinGraph_DispTrace (Trace_5, clWhite);
  if CbxTrace6.Checked then LinGraph_DispTrace (Trace_6, clAqua);
  if CbxTrace7.Checked then LinGraph_DispTrace (Trace_7, clLime);
  if CbxTrace8.Checked then LinGraph_DispTrace (Trace_8, clWhite);
  if CbxTrace9.Checked then LinGraph_DispTrace (Trace_9, clAqua);
  if CbxTrace10.Checked then LinGraph_DispTrace (Trace_10, clCream);
  if CbxTrace11.Checked then LinGraph_DispTrace (Trace_11, clCream);
  end;
end;

procedure LinGraph_Clear;
var i: integer;
begin
  for i := 1 to nPoints-1 do  // shift traces 1 point left
  begin
    Trace_1[i]:= 0;
    Trace_2[i]:= 0;
    Trace_3[i]:= 0;
    Trace_4[i]:= 0;
    Trace_5[i]:= 0;
    Trace_6[i]:= 0;
    Trace_7[i]:= 0;
    Trace_8[i]:= 0;
    Trace_9[i]:= 0;
    Trace_10[i]:= 0;
    Trace_11[i]:= 0;
  end;
end;

procedure LinGraph_Update;
var i: integer;
begin
  for i := 1 to nPoints-1 do  // shift traces 1 point left
  begin
    Trace_1[i]:=  Trace_1[i+1];
    Trace_2[i]:=  Trace_2[i+1];
    Trace_3[i]:=  Trace_3[i+1];
    Trace_4[i]:=  Trace_4[i+1];
    Trace_5[i]:=  Trace_5[i+1];
    Trace_6[i]:=  Trace_6[i+1];
    Trace_7[i]:=  Trace_7[i+1];
    Trace_8[i]:=  Trace_8[i+1];
    Trace_9[i]:=  Trace_9[i+1];
    Trace_10[i]:= Trace_10[i+1];
    Trace_11[i]:= Trace_11[i+1];
  end;
  LinGraph_Init;
  Trace_1[nPoints]:=  Hour * 15 + 5;
  Trace_2[nPoints]:=  (avTPassCenter             * Scale_2)  + Offset_2;
  Trace_3[nPoints]:=  (avAPeakCenter             * Scale_3)  + Offset_3;
  Trace_4[nPoints]:=  (avTPassRim_1              * Scale_4)  + Offset_4;
  Trace_5[nPoints]:=  (avAPeakRim_1              * Scale_5)  + Offset_5;
  Trace_6[nPoints]:=  (avTPassRim_2              * Scale_6)  + Offset_6;
  Trace_7[nPoints]:=  (avAPeakRim_2              * Scale_7)  + Offset_7;
  Trace_8[nPoints]:=  (avEllipseShortAxis_mm     * Scale_8)  + Offset_8;
  Trace_9[nPoints]:=  (avPrecessionAngle_Degrees * Scale_9)  + Offset_9;
  Trace_10[nPoints]:= (avPrecessionSpeed         * Scale_10) + Offset_10;
  Trace_11[nPoints]:= (avDrivePulseWidth         * Scale_11) + Offset_11;
  LinGraph_Refresh;
end;

end.

